"""
工具注册与调用
"""

# 统一导入所有工具类，兼容包内与单文件调试
try:
    from .tools import (
        FileRead, FileWrite, FileEdit,
        Ls,
        Glob, Grep,
        Bash,
        GitStatus, GitLog, GitDiff, GitBranch, GitPull, GitPush
    )
except ImportError:
    # 兼容直接运行本文件的情况
    from ...features.tools import (
        FileRead, FileWrite, FileEdit,
        Ls,
        Glob, Grep,
        Bash,
        GitStatus, GitLog, GitDiff, GitBranch, GitPull, GitPush
    )

class ToolRegistry:
    def __init__(self):
        self.tools = {}
        # 注册基础工具
        for tool in [FileRead(), FileWrite(), FileEdit(), Ls(), Glob(), Grep(), Bash(),
                GitStatus(), GitLog(), GitDiff(), GitBranch(), GitPull(), GitPush()]:
            self.register(tool)

    def register(self, tool):
        self.tools[tool.name] = tool

    def get(self, name):
        return self.tools.get(name)

def run_tool(tool_name, args):
    registry = ToolRegistry()
    tool = registry.get(tool_name)
    if not tool:
        return f"工具 {tool_name} 未注册"
    try:
        return tool.execute(**args)
    except Exception as e:
        return f"工具执行失败: {e}"


"""核心层 - 提供基础组件和抽象接口

包含模型抽象、上下文管理、安全框架、工具系统基础和通用工具。
"""

from .models import get_llm_interface, LLMInterfaceBase
from .context import ContextEngine
from .security import SecurityManager, SecurityLevel
from .tools import ToolRegistry, run_tool
from .tools import BaseTool

__all__ = [
    'get_llm_interface',
    'LLMInterfaceBase',
    'ContextEngine',
    'SecurityManager',
    'SecurityLevel',
    'ToolRegistry',
    'run_tool',
    'BaseTool',
]


"""模型抽象和适配器

提供各种 LLM 接口的抽象和实现。
"""

from .llm_interface import (
    LLMInterfaceBase,
    OpenAIBasedLLM,
    OpenAILLM,
    VolcEngineLLM,
    AnthropicLLM,
    get_llm_interface,
    parse_llm_tool_call,
    get_all_tool_descriptions
)

__all__ = [
    'LLMInterfaceBase',
    'OpenAIBasedLLM',
    'OpenAILLM',
    'VolcEngineLLM',
    'AnthropicLLM',
    'get_llm_interface',
    'parse_llm_tool_call',
    'get_all_tool_descriptions'
]


"""
Bash 命令相关工具
"""
import subprocess

from .base import BaseTool


class Bash(BaseTool):
    name = "Bash"
    description = "在 macOS 终端执行 Shell 命令（高风险，需严格确认）"
    arguments = {"command": "Shell 命令字符串"}
    def execute(self, command):
        try:
            result = subprocess.run(command, shell=True, capture_output=True, text=True, timeout=30)
            return result.stdout or result.stderr
        except Exception as e:
            return f"执行出错: {e}"



import os
import json
import warnings
from abc import ABC, abstractmethod
from typing import Any, Dict, Optional

import yaml

try:
    from openai import OpenAI, OpenAIError
    import httpx
except ImportError:
    OpenAI = None # type: ignore
    OpenAIError = None # type: ignore
    httpx = None # type: ignore

# 工具描述自动注入

def get_all_tool_descriptions() -> str:
    """
    自动收集所有注册工具的描述信息，拼接为 prompt 注入字符串。
    """
    from ..tools.tool_registry import ToolRegistry
    registry = ToolRegistry()
    descs = []
    for tool in registry.tools.values():
        descs.append(f"工具名: {tool.name}\n用途: {getattr(tool, 'description', '')}\n参数: {getattr(tool, 'arguments', {})}\n")
    return "\n".join(descs)

def parse_llm_tool_call(llm_output: str):
    """
    尝试解析 LLM 输出的结构化工具调用意图（JSON 格式）。
    成功则自动调用工具并返回结果，否则返回 None。
    """
    try:
        data = json.loads(llm_output)
        if "tool_call" in data:
            tool_name = data["tool_call"].get("tool_name")
            args = data["tool_call"].get("args", {})
            from ..tools.tool_registry import run_tool
            result = run_tool(tool_name, args)
            return f"[工具 {tool_name} 调用结果]\n{result}"
    except (json.JSONDecodeError, KeyError, TypeError) as error:
        # 处理特定的异常类型
        if isinstance(error, json.JSONDecodeError):
            # JSON 解析错误
            pass
        elif isinstance(error, (KeyError, TypeError)):
            # 数据结构或类型错误
            pass
    return None

class LLMInterfaceBase(ABC):
    """
    LLM API 抽象基类，定义通用接口。
    """
    def __init__(self, config: Optional[Dict[str, Any]] = None):
        self.config = config or {}
        self.api_key = self.config.get("api_key")
        self.model = self.config.get("model")
        # Use 'endpoint' instead of 'api_base' for consistency with config
        self.api_base = self.config.get("endpoint") 
        self.extra_params = self.config.get("extra_params", {})
        self.client = None # Initialize client later
        
        # 从配置中获取 SSL 验证设置
        self.verify_ssl = self._parse_verify_ssl_config()
        
    def _parse_verify_ssl_config(self) -> bool:
        """从配置中解析 SSL 验证设置"""
        # 兼容 llm 嵌套和顶层 verify_ssl
        if "verify_ssl" in self.config:
            _verify_ssl_raw = self.config["verify_ssl"]
        elif "llm" in self.config and isinstance(self.config["llm"], dict) and "verify_ssl" in self.config["llm"]:
            _verify_ssl_raw = self.config["llm"]["verify_ssl"]
        else:
            return True  # 默认启用 SSL 验证
            
        # 处理不同类型的值
        if isinstance(_verify_ssl_raw, bool):
            return _verify_ssl_raw
        elif isinstance(_verify_ssl_raw, str):
            return _verify_ssl_raw.strip().lower() in ("true", "1", "yes")
        else:
            return bool(_verify_ssl_raw)
    
    def _handle_api_error(self, error: Exception, provider_name: str = "API") -> str:
        """通用 API 错误处理逻辑"""
        error_msg = str(error)
        
        # 网络连接错误处理
        if any(keyword in error_msg.lower() for keyword in ["connect", "connection", "timeout"]):
            try:
                import requests
                # 测试基本网络连接
                requests.get("https://www.baidu.com", timeout=5)
                # 如果基本网络正常，可能是API端点问题
                api_key_preview = self.api_key[:5] + "..." if self.api_key else "未设置"
                return (f"Error: Connection error. 无法连接到{provider_name}服务器({self.api_base})。请检查:\n"
                        f"1. API端点配置是否正确\n"
                        f"2. 网络设置（防火墙、代理）是否允许访问该地址\n"
                        f"3. API密钥({api_key_preview})是否正确且有效\n"
                        f"4. 模型ID({self.model})是否正确")
            except ImportError:
                return (f"Error: Connection error. 无法连接到{provider_name}服务器({self.api_base})。\n"
                       f"(提示: 安装 'requests' 库以获取更详细的网络诊断信息: pip install requests)")
            except Exception:
                # 基本网络连接也有问题
                return f"Error: Connection error. 网络连接异常，请检查您的网络连接是否正常。"
        
        # API密钥错误处理
        elif any(keyword in error_msg.lower() for keyword in ["key", "auth", "认证"]):
            api_key_preview = self.api_key[:5] + "..." if self.api_key else "未设置"
            return (f"Error: API key error. API密钥认证失败。请检查:\n"
                    f"1. API密钥({api_key_preview})是否正确\n"
                    f"2. 该密钥是否有权限访问模型({self.model})")
        
        # SSL证书错误处理
        elif any(keyword in error_msg.lower() for keyword in ["certificate", "ssl", "证书"]):
            return "Error: SSL证书验证失败。请在配置文件中添加 'verify_ssl: false' 关闭证书验证（仅用于测试环境）。"
        
        # 其他错误
        return f"Error: {error_msg}"

    @abstractmethod
    def generate(self, prompt: str, **kwargs) -> str:
        """
        生成 LLM 响应 (通常用于非聊天模型)
        """
        pass

    @abstractmethod
    def chat(self, messages: list, **kwargs) -> str:
        """
        支持多轮对话的接口
        """
        pass

    @classmethod
    def from_config_file(cls, config_path: str):
        """
        从 YAML/TOML 配置文件加载参数
        """
        if config_path.endswith(".yaml") or config_path.endswith(".yml"):
            with open(config_path, "r", encoding="utf-8") as f:
                config = yaml.safe_load(f)
        else:
            raise NotImplementedError("只支持 YAML 配置文件")
        # Extract the llm part of the config if present
        llm_config = config.get("llm", config) 
        return cls(llm_config)

# 创建一个 OpenAI 兼容 API 的中间基类
class OpenAIBasedLLM(LLMInterfaceBase):
    """使用 OpenAI 兼容 API 的 LLM 基类"""
    DEFAULT_BASE_URL = None  # 子类应该覆盖这个值
    
    def __init__(self, config: Optional[Dict[str, Any]] = None):
        super().__init__(config)
        if not OpenAI:
            raise ImportError("openai 库未安装，请运行 'pip install openai>=1.0'")
        
        # 初始化客户端
        self._init_client()
    
    def _init_client(self):
        """初始化 OpenAI 客户端"""
        # 确保 base_url 格式正确（移除末尾斜杠）
        base_url = self.api_base or self.DEFAULT_BASE_URL
        if base_url and base_url.endswith('/'):
            base_url = base_url[:-1]
            
        # 检查 API 密钥
        if not self.api_key:
            raise ValueError(f"未在配置文件中找到 API Key (api_key)")
            
        # 处理 SSL 验证
        if not self.verify_ssl and httpx:
            try:
                import urllib3
                # 禁用 SSL 证书验证的警告
                urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
                warnings.warn("警告: SSL 证书验证已禁用，仅用于测试环境。生产环境请启用 SSL 验证。")
                
                # 创建带有 SSL 验证设置的客户端
                self.client = OpenAI(
                    base_url=base_url,
                    api_key=self.api_key,
                    http_client=httpx.Client(verify=self.verify_ssl)
                )
            except ImportError:
                # 如果没有 urllib3，仍然创建客户端但不禁用警告
                self.client = OpenAI(
                    base_url=base_url,
                    api_key=self.api_key,
                    http_client=httpx.Client(verify=self.verify_ssl)
                )
        else:
            # 正常创建客户端（启用 SSL 验证）
            self.client = OpenAI(
                base_url=base_url,
                api_key=self.api_key
            )
    
    def generate(self, prompt: str, **kwargs) -> str:
        """使用聊天接口模拟 generate 功能"""
        messages = [{"role": "user", "content": prompt}]
        return self.chat(messages, **kwargs)
    
    def _process_completion_response(self, completion, stream=False):
        """处理 API 返回的 completion 对象"""
        if stream:
            # 处理流式响应，返回生成器
            def response_generator():
                for chunk in completion:
                    if chunk.choices and chunk.choices[0].delta and chunk.choices[0].delta.content:
                        yield chunk.choices[0].delta.content
            return response_generator()
        else:
            # 处理普通响应
            if hasattr(completion, "choices") and completion.choices and len(completion.choices) > 0:
                return completion.choices[0].message.content.strip()
            return "Error: 模型未返回有效内容"
    
    def chat(self, messages: list, **kwargs):
        """实现通用的 chat 接口
        
        当 stream=True 时，返回生成器而不是字符串
        """
        try:
            # 获取参数，优先使用传入的参数，其次使用配置中的参数，最后使用默认值
            max_tokens = kwargs.get("max_tokens", self.config.get("max_tokens", 2048))
            temperature = kwargs.get("temperature", self.config.get("temperature", 0.7))
            stream = kwargs.get("stream", False)
            
            # 创建请求参数
            params = {
                "model": self.model,
                "messages": messages,
                "max_tokens": max_tokens,
                "temperature": temperature,
                "stream": stream,
                **self.extra_params  # 添加额外参数
            }
            
            # 移除值为 None 的参数
            params = {k: v for k, v in params.items() if v is not None}
            
            # 调用 API
            completion = self.client.chat.completions.create(**params)
            
            # 处理响应
            return self._process_completion_response(completion, stream)
            
        except OpenAIError as error:
            # 使用通用错误处理
            error_msg = self._handle_api_error(error, "OpenAI")
            if stream:
                # 对于流式输出，将错误消息包装成生成器返回
                def error_generator():
                    yield error_msg
                return error_generator()
            return error_msg
        except Exception as error:
            # 处理其他未预期的错误
            error_msg = f"Error: Unexpected error: {str(error)}"
            if stream:
                # 对于流式输出，将错误消息包装成生成器返回
                def error_generator():
                    yield error_msg
                return error_generator()
            return error_msg

# OpenAI 子类
class OpenAILLM(OpenAIBasedLLM):
    """OpenAI API 接口"""
    def __init__(self, config: Optional[Dict[str, Any]] = None):
        super().__init__(config)

# 火山引擎 LLM 接口 (使用 OpenAI SDK)
class VolcEngineLLM(OpenAIBasedLLM):
    """火山引擎 LLM 接口，使用 OpenAI 兼容 API"""
    DEFAULT_BASE_URL = "https://ark.cn-beijing.volces.com/api/v3"

    def __init__(self, config: Optional[Dict[str, Any]] = None):
        # 从环境变量获取 API Key，如果配置中也提供了，优先使用配置中的
        config = config or {}
        if not config.get("api_key"):
            api_key = os.environ.get("ARK_API_KEY")
            if api_key:
                config["api_key"] = api_key
        
        # 初始化父类
        super().__init__(config)
        
        # 设置环境变量，以防其他地方需要
        if self.api_key and not os.environ.get("ARK_API_KEY"):
            try:
                os.environ["ARK_API_KEY"] = self.api_key
            except (TypeError, ValueError):
                pass
        
        # 模型 ID 是必须的
        if not self.model:
            raise ValueError("未在配置中指定火山引擎模型 (model)。")
    
    def chat(self, messages: list, **kwargs) -> str:
        # 火山引擎的默认 max_tokens 与 OpenAI 不同
        if "max_tokens" not in kwargs and "max_tokens" not in self.config:
            kwargs["max_tokens"] = 500  # 火山引擎默认值
            
        # 使用父类的 chat 方法
        try:
            return super().chat(messages, **kwargs)
        except Exception as error:
            # 处理火山引擎特有的错误
            error_msg = str(error)
            if "certificate verify failed" in error_msg.lower() or "ssl" in error_msg.lower():
                return "Error: SSL证书验证失败。请在配置文件中添加 'verify_ssl: false' 关闭证书验证（仅用于测试环境）。"
            return self._handle_api_error(error, "火山引擎")


# 预留：Anthropic、其他 LLM 子类可类似扩展

# 非 OpenAI 兼容的 LLM 接口
class AnthropicLLM(LLMInterfaceBase):
    """人类智慧公司 Anthropic 的 Claude 模型接口"""
    def __init__(self, config: Optional[Dict[str, Any]] = None):
        super().__init__(config)
        # 检查必要的配置
        if not self.api_key:
            raise ValueError("未在配置文件中找到 Anthropic API Key (api_key)")
        
        # TODO: 使用官方 anthropic 库初始化客户端
        try:
            import anthropic
            self.client = anthropic.Anthropic(api_key=self.api_key)
        except ImportError:
            raise ImportError("anthropic 库未安装，请运行 'pip install anthropic'")

    def generate(self, prompt: str, **kwargs) -> str:
        """使用聊天接口模拟 generate 功能"""
        messages = [{"role": "user", "content": prompt}]
        return self.chat(messages, **kwargs)

    def chat(self, messages: list, **kwargs) -> str:
        try:
            # 获取参数
            max_tokens = kwargs.get("max_tokens", self.config.get("max_tokens", 1024))
            temperature = kwargs.get("temperature", self.config.get("temperature", 0.7))
            model = self.model or "claude-3-opus-20240229"
            
            # 转换消息格式（如果需要）
            # TODO: 实现消息格式转换逻辑
            
            # 调用 API
            response = self.client.messages.create(
                model=model,
                messages=messages,
                max_tokens=max_tokens,
                temperature=temperature
            )
            
            # 处理响应
            return response.content[0].text
        except ImportError as error:
            return f"Error: Anthropic 库导入错误: {error}"
        except ValueError as error:
            return f"Error: 参数错误: {error}"
        except ConnectionError as error:
            return f"Error: 连接错误: {error}"
        except AttributeError as error:
            return f"Error: 属性错误: {error}"
        except Exception as error:
            # 使用通用错误处理，仅在其他异常都不匹配时使用
            return self._handle_api_error(error, "Anthropic")


# 中国本土 LLM 接口类的基类
class ChineseLLMBase(LLMInterfaceBase):
    """中国本土 LLM 接口的基类，包含共享功能"""
    
    def __init__(self, config: Optional[Dict[str, Any]] = None):
        super().__init__(config)
        # 检查必要的配置
        if not self.api_key:
            raise ValueError(f"未在配置文件中找到 {self.__class__.__name__} API Key (api_key)")
        
        # 初始化客户端
        self._init_client()
    
    def _init_client(self):
        """初始化客户端，子类应覆盖此方法"""
        # 这是一个抽象方法，由子类实现
    
    def generate(self, prompt: str, **kwargs) -> str:
        """使用聊天接口模拟 generate 功能"""
        messages = [{"role": "user", "content": prompt}]
        return self.chat(messages, **kwargs)


class WenxinLLM(ChineseLLMBase):
    """百度文心千帆 LLM 接口"""
    
    def _init_client(self):
        """初始化百度文心千帆客户端"""
        try:
            import qianfan
            self.client = qianfan.Qianfan(ak=self.api_key, sk=self.config.get("secret_key"))
        except ImportError:
            raise ImportError("qianfan 库未安装，请运行 'pip install qianfan'")

    def chat(self, messages: list, **kwargs) -> str:
        try:
            # 获取参数
            model = self.model or "ERNIE-Bot-4"
            
            # 调用 API
            response = self.client.chat(model=model, messages=messages)
            
            # 处理响应
            return response.result
        except ImportError as error:
            return f"Error: 百度文心库导入错误: {error}"
        except ValueError as error:
            return f"Error: 参数错误: {error}"
        except ConnectionError as error:
            return f"Error: 连接错误: {error}"
        except AttributeError as error:
            return f"Error: 属性错误: {error}"
        except Exception as error:
            # 仅在其他异常都不匹配时使用通用错误处理
            return self._handle_api_error(error, "百度文心")


class TongyiLLM(ChineseLLMBase):
    """阿里通义千问 LLM 接口"""
    
    def _init_client(self):
        """初始化阿里通义千问客户端"""
        try:
            import dashscope
            dashscope.api_key = self.api_key
            self.client = dashscope
        except ImportError:
            raise ImportError("dashscope 库未安装，请运行 'pip install dashscope'")

    def chat(self, messages: list, **kwargs) -> str:
        try:
            # 获取参数
            model = self.model or "qwen-max"
            
            # 调用 API
            from dashscope import Generation
            response = Generation.call(
                model=model,
                messages=messages,
                result_format='message',
            )
            
            # 处理响应
            if response.status_code == 200:
                return response.output.choices[0].message.content
            else:
                return f"Error: API 调用失败 (状态码: {response.status_code})"
        except ImportError as error:
            return f"Error: 阿里通义库导入错误: {error}"
        except ValueError as error:
            return f"Error: 参数错误: {error}"
        except ConnectionError as error:
            return f"Error: 连接错误: {error}"
        except AttributeError as error:
            return f"Error: 属性错误: {error}"
        except Exception as error:
            # 仅在其他异常都不匹配时使用通用错误处理
            return self._handle_api_error(error, "阿里通义")


class QwenLLM(OpenAIBasedLLM):
    """阿里千问 LLM 接口 (使用 OpenAI 兼容 API)"""
    
    DEFAULT_BASE_URL = "https://dashscope.aliyuncs.com/compatible-mode/v1"
    
    def __init__(self, config: Optional[Dict[str, Any]] = None):
        super().__init__(config)
        
        # 确保必要的配置存在
        if not self.api_key:
            raise ValueError("未在配置中指定千问 API Key (api_key)。")
        
        # 默认模型
        if not self.model:
            self.model = "qwen-plus"
    
    def _init_client(self):
        """初始化 OpenAI 客户端"""
        if not OpenAI:
            raise ImportError("未安装 OpenAI 库，请运行 'pip install openai'。")
        
        # 使用用户提供的 API 基础地址或默认值
        base_url = self.api_base or self.DEFAULT_BASE_URL
        
        # 创建 OpenAI 客户端
        try:
            self.client = OpenAI(
                api_key=self.api_key,
                base_url=base_url,
            )
        except Exception as e:
            raise RuntimeError(f"初始化千问 API 客户端失败: {e}")
    
    def chat(self, messages: list, **kwargs):
        stream = kwargs.get("stream", False)
        
        try:
            # 初始化客户端（如果还没有初始化）
            if not self.client:
                self._init_client()
            
            # 准备调用参数
            call_kwargs = {
                "model": self.model,
                "messages": messages,
                "stream": stream,
            }
            
            # 对于 Qwen3 模型，添加 enable_thinking 参数
            if "qwen3" in self.model.lower() and not stream:
                call_kwargs["extra_body"] = {"enable_thinking": False}
            
            # 添加其他用户参数
            for key, value in kwargs.items():
                if key not in ["stream"]:
                    call_kwargs[key] = value
            
            # 调用 API
            response = self.client.chat.completions.create(**call_kwargs)
            
            # 处理响应
            if stream:
                # 流式输出模式
                def response_generator():
                    for chunk in response:
                        if hasattr(chunk, "choices") and chunk.choices and hasattr(chunk.choices[0], "delta"):
                            content = chunk.choices[0].delta.content
                            if content:
                                yield content
                return response_generator()
            else:
                # 非流式输出模式
                return response.choices[0].message.content
        except Exception as e:
            # 处理错误
            error_msg = self._handle_api_error(e, "阿里千问")
            if stream:
                # 流式输出模式下，将错误消息包装为生成器返回
                def error_generator():
                    yield error_msg
                return error_generator()
            else:
                return error_msg


class ZhipuAILLM(ChineseLLMBase):
    """智谱AI LLM 接口"""
    
    def _init_client(self):
        """初始化智谱AI客户端"""
        try:
            import zhipuai
            self.client = zhipuai.ZhipuAI(api_key=self.api_key)
        except ImportError:
            raise ImportError("zhipuai 库未安装，请运行 'pip install zhipuai'")

    def chat(self, messages: list, **kwargs) -> str:
        try:
            # 获取参数
            model = self.model or "glm-4"
            temperature = kwargs.get("temperature", self.config.get("temperature", 0.7))
            
            # 调用 API
            response = self.client.chat.completions.create(
                model=model,
                messages=messages,
                temperature=temperature
            )
            
            # 处理响应
            return response.choices[0].message.content
        except ImportError as error:
            return f"Error: 智谱AI库导入错误: {error}"
        except ValueError as error:
            return f"Error: 参数错误: {error}"
        except ConnectionError as error:
            return f"Error: 连接错误: {error}"
        except AttributeError as error:
            return f"Error: 属性错误: {error}"
        except Exception as error:
            # 仅在其他异常都不匹配时使用通用错误处理
            return self._handle_api_error(error, "智谱AI")


# LLM 工厂函数，根据配置动态选择接口
def get_llm_interface(config: Dict[str, Any]) -> LLMInterfaceBase:
    # config 必须是 llm dict（包含 provider、api_key、model、endpoint）
    provider = config.get("provider", "openai").lower()

    if provider == "openai":
        return OpenAILLM(config)
    elif provider == "volcengine":
        return VolcEngineLLM(config)
    elif provider == "anthropic":
        return AnthropicLLM(config)
    elif provider == "wenxin":
        return WenxinLLM(config)
    elif provider == "tongyi":
        return TongyiLLM(config)
    elif provider == "zhipuai":
        return ZhipuAILLM(config)
    elif provider == "qwen":
        return QwenLLM(config)
    else:
        raise ValueError(f"Unsupported LLM provider: {provider}")


from rich.markdown import Markdown
from rich.panel import Panel
from rich.progress import Progress, SpinnerColumn, TextColumn

# 使用 rich 库自己的控制台，避免循环导入
from rich.console import Console
console = Console()


class ChatCommand:
    def __init__(self, agent, verbose: bool = False, stream: bool = False):
        self.agent = agent
        self.verbose = verbose
        self.stream = stream
        self.history = []  # [{role: "user"/"assistant", content: str}]

    def add_message(self, role: str, content: str):
        """添加消息到历史记录"""
        self.history.append({"role": role, "content": content})
        # 如果需要在控制台显示消息，可以取消下面的注释
        # if self.verbose:
        #     display_prefix = "[用户]" if role == "user" else "[nezha]"
        #     console.print(f"{display_prefix}: {content}")

    def interactive_loop(self, initial_message=None):
        """处理交互式对话循环"""
        if initial_message:
            self.add_message("user", initial_message)
        
        while True:
            if not self.history:  # 如果没有初始消息，先获取用户输入
                user_input = input("\n请输入你的问题 (输入'exit'退出): ").strip()
                if user_input.lower() in ['exit', 'quit', '退出']:
                    break
                self.add_message("user", user_input)
            
            # 调用agent处理对话
            try:
                if not self.stream:
                    # 非流式模式
                    # 添加加载指示器
                    with Progress(
                        SpinnerColumn(),
                        TextColumn("[progress.description]{task.description}"),
                        transient=True,  # 完成时移除进度显示
                        console=console   # 使用相同的控制台
                    ) as progress:
                        progress.add_task(description="思考中...", total=None)  # 不确定的任务
                        response = self.agent.plan_chat(self.history, self.verbose)
                    
                    self.add_message("assistant", response)
                    console.print(Panel(Markdown(response), title="nezha", border_style="cyan"))
                else:
                    # 流式输出模式
                    console.print("\n[bold cyan]nezha:[/bold cyan]")
                    console.print("[dim](按Ctrl+C可中断生成)[/dim]")
                    response_generator = self.agent.plan_chat(self.history, verbose=self.verbose, stream=True)
                    response_text = ""
                    try:
                        for chunk in response_generator:
                            console.print(chunk, end="")
                            response_text += chunk
                    except KeyboardInterrupt:
                        console.print("\n\n[bold red][生成已被用户中断][/bold red]")
                    finally:
                        console.print()  # 换行
                        # 即使被中断，也将已生成的内容添加到历史记录中
                        self.add_message("assistant", response_text)
                        # 在流式输出后显示分隔线
                        console.print("\n" + "-"*50)
            except (ValueError, TypeError) as error:
                # 处理可预期的错误类型
                console.print(Panel(f"[bold]对话处理出错:[/bold] {error}", title="错误", border_style="red"))
                if self.verbose:
                    import traceback
                    console.print(traceback.format_exc())
            except Exception as error:
                # 处理未预期的错误
                console.print(Panel(f"[bold]未预期的错误:[/bold] {error}", title="错误", border_style="red"))
                if self.verbose:
                    import traceback
                    console.print(traceback.format_exc())
            
            # 获取下一轮用户输入
            user_input = input("\n请输入你的问题 (输入'exit'退出): ").strip()
            if user_input.lower() in ['exit', 'quit', '退出']:
                break
            self.add_message("user", user_input)

    def run(self, initial_message=None):
        """运行对话命令"""
        try:
            self.interactive_loop(initial_message)
        except KeyboardInterrupt:
            # 处理用户中断
            pass
        except KeyError as error:
            # 处理键错误（如访问不存在的字典键）
            console.print(Panel(f"[bold]对话数据错误:[/bold] {error}", title="错误", border_style="red"))
            if self.verbose:
                import traceback
                console.print(traceback.format_exc())
        except Exception as error:
            # 处理其他未预期的错误
            console.print(Panel(f"[bold]对话过程中发生错误:[/bold] {error}", title="错误", border_style="red"))
            if self.verbose:
                import traceback
                console.print(traceback.format_exc())
        return "对话已完成"


"""
Nezha Agent 工具集

提供文件操作、搜索、导航、Git 操作等工具
"""

# 导出所有工具类
from .file_io import FileRead, FileWrite, FileEdit
from .navigation import Ls
from .search import Glob, Grep
from .shell import Bash
from .git_tools import GitStatus, GitLog, GitDiff, GitBranch, GitPull, GitPush

__all__ = [
    'FileRead', 'FileWrite', 'FileEdit',
    'Ls',
    'Glob', 'Grep',
    'Bash',
    'GitStatus', 'GitLog', 'GitDiff', 'GitBranch', 'GitPull', 'GitPush'
]



import os
from pathlib import Path
from typing import Any, Dict, List, Optional


class PlanCommand:
    def __init__(self, agent, context_engine, verbose: bool = False, stream: bool = False, output_file: Optional[Path] = None):
        self.agent = agent
        self.context_engine = context_engine
        self.verbose = verbose
        self.stream = stream
        self.output_file = output_file or Path("plan_output.md")
        self.history: List[Dict[str, str]] = []  # [{role: "user"/"assistant", content: str}]

    def add_message(self, role: str, content: str):
        self.history.append({"role": role, "content": content})
        if self.verbose:
            prefix = "[用户]" if role == "user" else "[AI]"
            print(f"{prefix}: {content}")

    def interactive_loop(self, initial_requirement: str):
        self.add_message("user", initial_requirement)
        while True:
            # 与agent多轮对话
            if not self.stream:
                # 非流式输出模式
                response = self.agent.plan_chat(self.history, verbose=self.verbose)
                self.add_message("assistant", response)
                print("\nnezha建议:\n", response)
            else:
                # 流式输出模式
                print("\nnezha建议:")
                print("(按Ctrl+C可中断生成)")
                response_generator = self.agent.plan_chat(self.history, verbose=self.verbose, stream=True)
                response_text = ""
                try:
                    for chunk in response_generator:
                        print(chunk, end="", flush=True)
                        response_text += chunk
                except KeyboardInterrupt:
                    print("\n\n[生成已被用户中断]")
                finally:
                    print()  # 换行
                    # 即使被中断，也将已生成的内容添加到历史记录中
                    self.add_message("assistant", response_text)
                
            user_input = input("请输入补充/修改需求，或直接回车结束: ").strip()
            if not user_input:
                break
            self.add_message("user", user_input)
        return self.generate_plan_markdown()

    def generate_plan_markdown(self) -> str:
        # 结构化输出Markdown计划
        plan_sections = [msg["content"] for msg in self.history if msg["role"] == "assistant"]
        markdown = "# 项目规划计划\n\n"
        for idx, section in enumerate(plan_sections, 1):
            markdown += f"## 步骤{idx}\n{section}\n\n"
        if self.verbose:
            # print("\n最终计划Markdown:\n", markdown)
            pass
        return markdown

    def save_plan(self, markdown: str):
        with open(self.output_file, "w", encoding="utf-8") as f:
            f.write(markdown)
        if self.verbose:
            # print(f"计划已保存到: {self.output_file}")
            pass

    def run(self, initial_requirement: str):        
        markdown = self.interactive_loop(initial_requirement)
        self.save_plan(markdown)
        # print(f"\n规划已完成，计划文档输出至: {self.output_file}")
        return markdown


"""
安全工具集成示例 - 展示如何将安全层与工具系统集成
"""

from typing import Any, Dict, Optional

from nezha_agent.security import SecurityLevel, ToolRiskLevel, security_manager
from .base import BaseTool
from .file_io import FileEdit, FileWrite
from .shell import Bash


class SecureFileWrite(FileWrite):
    """安全版文件写入工具，集成了安全检查"""
    
    def execute(self, path: str, content: str) -> str:
        # 检查路径是否在允许范围内
        if not security_manager.is_path_allowed(path):
            return f"安全错误: 路径 '{path}' 不在允许操作的范围内"
        
        # 请求用户确认
        details = {
            "路径": path,
            "内容长度": f"{len(content)} 字符",
            "内容预览": f"{content[:100]}{'...' if len(content) > 100 else ''}"
        }
        
        if not security_manager.confirm_action(
            message=f"即将写入文件 '{path}'",
            risk_level=ToolRiskLevel.MEDIUM,
            details=details
        ):
            return "操作已取消"
        
        # 调用原始方法执行写入
        return super().execute(path, content)


class SecureFileEdit(FileEdit):
    """安全版文件编辑工具，集成了安全检查"""
    
    def execute(self, path: str, instructions: str) -> str:
        # 检查路径是否在允许范围内
        if not security_manager.is_path_allowed(path):
            return f"安全错误: 路径 '{path}' 不在允许操作的范围内"
        
        # 请求用户确认
        details = {
            "路径": path,
            "编辑指令": instructions
        }
        
        if not security_manager.confirm_action(
            message=f"即将编辑文件 '{path}'",
            risk_level=ToolRiskLevel.MEDIUM,
            details=details
        ):
            return "操作已取消"
        
        # 调用原始方法执行编辑
        return super().execute(path, instructions)


class SecureBash(Bash):
    """安全版Bash命令执行工具，集成了严格的安全检查"""
    
    def execute(self, command: str) -> str:
        # 检查工具是否被允许使用
        if not security_manager.is_tool_allowed("Bash", ToolRiskLevel.CRITICAL):
            return "安全错误: 当前安全级别下禁止执行Shell命令"
        
        # 请求用户确认（使用最高风险级别）
        details = {
            "命令": command
        }
        
        if not security_manager.confirm_action(
            message="即将执行Shell命令（高风险操作）",
            risk_level=ToolRiskLevel.CRITICAL,
            details=details
        ):
            return "操作已取消"
        
        # 调用原始方法执行命令
        return super().execute(command)


# 安全工具注册函数示例
def register_secure_tools(tool_registry):
    """将安全版工具注册到工具注册表中
    
    Args:
        tool_registry: 工具注册表实例
    """
    # 替换原始工具为安全版本
    tool_registry.register_tool(SecureFileWrite())
    tool_registry.register_tool(SecureFileEdit())
    tool_registry.register_tool(SecureBash())



"""上下文收集与管理

该模块负责收集和管理传递给LLM的上下文信息，包括目录结构、Git信息、指定文件内容和代码风格等。
实现token预算管理，当上下文过长时采用优先级策略选择信息。
"""

import os
import re
import subprocess
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

try:
    import git
    GIT_AVAILABLE = True
except ImportError:
    GIT_AVAILABLE = False

class ContextEngine:
    """上下文引擎类，负责收集和管理传递给LLM的上下文信息"""
    
    def __init__(self, max_tokens: int = 4000, working_dir: Optional[str] = None):
        """初始化上下文引擎
        
        Args:
            max_tokens: 上下文最大token数量
            working_dir: 工作目录，默认为当前目录
        """
        self.max_tokens = max_tokens
        self.working_dir = working_dir or os.getcwd()
        self.context_data = {}
        self.priority_levels = {
            "user_specified_files": 1,  # 用户明确指定的文件
            "git_changes": 2,          # Git变更的文件
            "key_files": 3,            # 项目关键文件
            "directory_structure": 4,   # 目录结构
            "code_style": 5,           # 代码风格
            "git_info": 6,             # Git信息
        }
    
    def collect(self, user_files: List[str] = None) -> Dict[str, Any]:
        """收集上下文信息
        
        Args:
            user_files: 用户指定的文件列表
            
        Returns:
            收集到的上下文信息字典
        """
        # 清空之前的上下文数据
        self.context_data = {}
        
        # 收集当前工作目录
        self.context_data["working_directory"] = self.working_dir
        
        # 收集目录结构
        self.collect_directory_structure()
        
        # 收集Git信息（如果可用）
        self.collect_git_info()
        
        # 收集用户指定的文件
        if user_files:
            self.collect_specified_files(user_files)
        
        # 收集项目关键文件
        self.collect_key_files()
        
        # 收集代码风格信息
        self.collect_code_style()
        
        # 应用token预算管理
        self.apply_token_budget()
        
        return self.context_data
    
    def collect_directory_structure(self, max_depth: int = 2):
        """收集目录结构
        
        Args:
            max_depth: 最大目录深度
        """
        try:
            # 尝试使用tree命令（如果可用）
            result = subprocess.run(
                ["tree", "-L", str(max_depth), "-d", self.working_dir],
                capture_output=True,
                text=True,
                check=False
            )
            if result.returncode == 0:
                self.context_data["directory_structure"] = result.stdout
                return
        except (subprocess.SubprocessError, FileNotFoundError):
            pass
        
        # 如果tree命令不可用，使用os.walk
        structure = []
        root_path = Path(self.working_dir)
        structure.append(str(root_path))
        
        for current_depth in range(1, max_depth + 1):
            for root, dirs, files in os.walk(self.working_dir):
                path = Path(root)
                rel_path = path.relative_to(self.working_dir)
                depth = len(rel_path.parts)
                
                if depth == current_depth - 1:
                    for d in sorted(dirs):
                        indent = "  " * depth
                        structure.append(f"{indent}├── {d}/")
        
        self.context_data["directory_structure"] = "\n".join(structure)
    
    def collect_git_info(self):
        """收集Git信息"""
        if not GIT_AVAILABLE:
            return
        
        try:
            repo = git.Repo(self.working_dir)
            git_info = {}
            
            # 获取工作区状态
            if not repo.bare:
                git_info["status"] = self._get_git_status(repo)
                
                # 获取最近的提交
                git_info["recent_commits"] = self._get_recent_commits(repo)
                
                # 获取变更的文件
                git_info["changed_files"] = self._get_changed_files(repo)
            
            self.context_data["git_info"] = git_info
        except (git.InvalidGitRepositoryError, git.NoSuchPathError):
            # 不是Git仓库或路径不存在
            pass
    
    def _get_git_status(self, repo):
        """获取Git状态"""
        return repo.git.status(porcelain=True)
    
    def _get_recent_commits(self, repo, count=5):
        """获取最近的提交"""
        commits = []
        for commit in repo.iter_commits(max_count=count):
            commits.append({
                "hash": commit.hexsha[:7],
                "message": commit.message.strip(),
                "author": f"{commit.author.name} <{commit.author.email}>",
                "date": commit.committed_datetime.isoformat(),
            })
        return commits
    
    def _get_changed_files(self, repo):
        """获取变更的文件"""
        changed_files = []
        for item in repo.index.diff(None):
            changed_files.append(item.a_path)
        return changed_files
    
    def collect_specified_files(self, file_paths: List[str]):
        """收集用户指定的文件内容
        
        Args:
            file_paths: 文件路径列表
        """
        specified_files = {}
        
        for file_path in file_paths:
            try:
                with open(file_path, 'r', encoding='utf-8') as f:
                    content = f.read()
                    specified_files[file_path] = content
            except (IOError, UnicodeDecodeError) as e:
                specified_files[file_path] = f"Error reading file: {str(e)}"
        
        self.context_data["user_specified_files"] = specified_files
    
    def collect_key_files(self):
        """收集项目关键文件"""
        key_files = [
            "README.md",
            "pyproject.toml",
            "setup.py",
            "requirements.txt",
            ".gitignore"
        ]
        
        found_key_files = {}
        
        for file_name in key_files:
            file_path = os.path.join(self.working_dir, file_name)
            if os.path.isfile(file_path):
                try:
                    with open(file_path, 'r', encoding='utf-8') as f:
                        content = f.read()
                        found_key_files[file_name] = content
                except (IOError, UnicodeDecodeError):
                    pass
        
        self.context_data["key_files"] = found_key_files
    
    def collect_code_style(self):
        """收集代码风格信息"""
        style_files = [
            ".flake8",
            ".pylintrc",
            "pyproject.toml",  # 可能包含black、isort等配置
            ".editorconfig",
            ".style.yapf",
        ]
        
        style_info = {}
        
        for file_name in style_files:
            file_path = os.path.join(self.working_dir, file_name)
            if os.path.isfile(file_path):
                try:
                    with open(file_path, 'r', encoding='utf-8') as f:
                        content = f.read()
                        style_info[file_name] = content
                except (IOError, UnicodeDecodeError):
                    pass
        
        self.context_data["code_style"] = style_info
    
    def apply_token_budget(self):
        """应用token预算管理，当上下文过长时采用优先级策略选择信息"""
        # 简单估算当前token数量（粗略估计：1个token约等于4个字符）
        current_tokens = self._estimate_tokens(self.context_data)
        
        if current_tokens <= self.max_tokens:
            return
        
        # 按优先级裁剪内容
        for category, priority in sorted(self.priority_levels.items(), key=lambda x: x[1], reverse=True):
            if category not in self.context_data:
                continue
                
            # 对于最高优先级的内容（用户指定的文件），尝试保留但可能需要截断
            if priority == 1 and isinstance(self.context_data[category], dict):
                for file_path, content in self.context_data[category].items():
                    if len(content) > 1000:  # 简单截断策略
                        self.context_data[category][file_path] = content[:500] + "\n...\n" + content[-500:]
            
            # 对于低优先级内容，如果仍然超出预算，则移除
            elif priority > 3:  # 目录结构及更低优先级
                current_tokens = self._estimate_tokens(self.context_data)
                if current_tokens > self.max_tokens:
                    self.context_data.pop(category, None)
    
    def _estimate_tokens(self, data) -> int:
        """粗略估计数据的token数量
        
        Args:
            data: 要估计的数据
            
        Returns:
            估计的token数量
        """
        if isinstance(data, str):
            return len(data) // 4
        elif isinstance(data, dict):
            return sum(self._estimate_tokens(k) + self._estimate_tokens(v) for k, v in data.items())
        elif isinstance(data, list):
            return sum(self._estimate_tokens(item) for item in data)
        else:
            return len(str(data)) // 4
    
    def format_context(self) -> str:
        """格式化上下文信息为字符串
        
        Returns:
            格式化后的上下文字符串
        """
        sections = []
        
        # 添加工作目录
        sections.append(f"当前工作目录: {self.context_data.get('working_directory', '未知')}\n")
        
        # 添加目录结构
        if "directory_structure" in self.context_data:
            sections.append("目录结构:\n```\n" + self.context_data["directory_structure"] + "\n```\n")
        
        # 添加Git信息
        if "git_info" in self.context_data:
            git_info = self.context_data["git_info"]
            git_section = ["Git信息:"]
            
            if "status" in git_info and git_info["status"]:
                git_section.append("状态:\n```\n" + git_info["status"] + "\n```\n")
            
            if "changed_files" in git_info and git_info["changed_files"]:
                git_section.append("变更文件:\n- " + "\n- ".join(git_info["changed_files"]) + "\n")
            
            if "recent_commits" in git_info and git_info["recent_commits"]:
                commits = git_info["recent_commits"]
                git_section.append("最近提交:")
                for commit in commits:
                    git_section.append(f"- {commit['hash']} - {commit['message']} ({commit['author']})")
            
            sections.append("\n".join(git_section) + "\n")
        
        # 添加用户指定的文件
        if "user_specified_files" in self.context_data and self.context_data["user_specified_files"]:
            files = self.context_data["user_specified_files"]
            sections.append("用户指定的文件:")
            for file_path, content in files.items():
                sections.append(f"文件: {file_path}\n```\n{content}\n```\n")
        
        # 添加关键文件
        if "key_files" in self.context_data and self.context_data["key_files"]:
            files = self.context_data["key_files"]
            sections.append("项目关键文件:")
            for file_name, content in files.items():
                sections.append(f"文件: {file_name}\n```\n{content}\n```\n")
        
        # 添加代码风格信息
        if "code_style" in self.context_data and self.context_data["code_style"]:
            sections.append("代码风格信息:")
            for file_name, content in self.context_data["code_style"].items():
                sections.append(f"配置文件: {file_name}\n```\n{content}\n```\n")
        
        return "\n".join(sections)


"""
Agent 核心逻辑类
"""
import os
from pathlib import Path
from typing import Any, Dict, Optional

import yaml

from ...core.context.context_engine import \
    ContextEngine  # Assuming ContextEngine provides context format
from ...core.models.llm_interface import LLMInterfaceBase, get_llm_interface
from ...core.security.security import SecurityManager

# 导入获取当前模型的函数
try:
    from ...ui.cli.cli import get_current_model
except ImportError:
    # 如果无法导入，定义一个空函数
    def get_current_model():
        return None

# 默认配置路径
DEFAULT_CONFIG_PATH = Path(os.path.expanduser("~/.config/nezha/config.yaml"))
# 默认内置配置路径
DEFAULT_BUILTIN_CONFIG_PATH = Path("config/default_config.yaml")

class NezhaAgent:
    def __init__(self, security_manager: SecurityManager, config_file: Optional[Path] = None, api_key: Optional[str] = None):
        self.security_manager = security_manager
        self.config_file = config_file
        self.config = self._load_config()
        self.api_key = api_key or os.environ.get("NEZHA_API_KEY")

        # 优先使用内存中的模型设置
        current_model = get_current_model()
        if current_model:
            llm_config = dict(current_model)
            # 合并推理参数
            file_llm_config = self.config.get("llm", {})
            for key, value in file_llm_config.items():
                if key not in llm_config and key not in ["provider", "model", "api_key", "endpoint"]:
                    llm_config[key] = value
            print(f"\n使用内存中的模型设置: {current_model['name']}")
        else:
            # 读取 llm 字段和 models 列表
            llm_section = self.config.get("llm", {})
            model_id = llm_section.get("model")
            user_models = self.config.get("models", [])
            # 延迟导入 PREDEFINED_MODELS，避免循环引用
            try:
                from ...ui.cli.cli import PREDEFINED_MODELS
            except ImportError:
                PREDEFINED_MODELS = []
            all_models = PREDEFINED_MODELS + user_models
            # 查找完整模型配置
            model_conf = next((m for m in all_models if m.get("id") == model_id), None)
            llm_config = dict(model_conf) if model_conf else {}
            # 检查是否为预置模型且 api_key 需覆盖
            if model_conf in PREDEFINED_MODELS:
                # 仅当 api_key 为 **** 或空时才覆盖
                key_to_use = self.api_key or llm_section.get("api_key")
                if not llm_config.get("api_key") or llm_config.get("api_key") == "****":
                    if key_to_use and key_to_use != "****":
                        llm_config["api_key"] = key_to_use
                    else:
                        raise RuntimeError(
                            "预置模型未配置有效 api_key！请通过以下方式之一设置：\n"
                            "1. 命令行参数 --api-key 传入\n"
                            "2. 设置环境变量 NEZHA_API_KEY（如：export NEZHA_API_KEY=你的key）\n"
                            "3. 在 config.yaml 的 llm.api_key 字段填写专属 key（如：llm:\n  api_key: 你的key）\n"
                            "   编辑方法：\n"
                            "   - 终端输入 nano ~/.config/nezha/config.yaml （nano简单易用，Ctrl+O 保存，Ctrl+X 退出）\n"
                            "   - 或 vim ~/.config/nezha/config.yaml （高级用户）\n"
                            "   - 或 code ~/.config/nezha/config.yaml （如果已安装 VSCode）\n"
                            "如需试用请联系管理员获取 key。"
                        )
            # 确保 model 字段存在
            if "id" in llm_config and "model" not in llm_config:
                llm_config["model"] = llm_config["id"]
            # 合并推理参数（如 temperature/max_tokens）
            for key in ["temperature", "max_tokens", "verify_ssl"]:
                if key in llm_section:
                    llm_config[key] = llm_section[key]
                    
        # 确保 verify_ssl 选项存在，默认为 True
        if "verify_ssl" not in llm_config:
            llm_config["verify_ssl"] = True
            
        # 初始化 LLM 接口
        try:
            self.llm_interface = get_llm_interface(llm_config)
            if not llm_config.get("verify_ssl", True):
                print("\n警告: SSL 证书验证已禁用。这可能会导致安全风险。")
        except Exception as e:
            raise RuntimeError(f"初始化 LLM 接口失败: {e}")

        # TODO: Initialize other components like tool registry based on config

    def _load_config(self) -> Dict[str, Any]:
        """加载配置文件。如果找不到则报错，要求用户先执行 nezha init 初始化。"""
        # 首先尝试从配置文件加载
        if self.config_file and Path(self.config_file).exists():
            try:
                with open(self.config_file, 'r', encoding='utf-8') as f:
                    config = yaml.safe_load(f) or {}
                    if config:
                        print(f"\n成功从 {self.config_file} 加载配置")
                        return config
            except Exception as e:
                print(f"Error loading config file {self.config_file}: {e}")
                raise RuntimeError(f"加载配置文件失败: {e}")

        # 尝试从默认内置配置文件加载
        if DEFAULT_BUILTIN_CONFIG_PATH.exists():
            try:
                with open(DEFAULT_BUILTIN_CONFIG_PATH, 'r', encoding='utf-8') as f:
                    default_config = yaml.safe_load(f) or {}
                    if default_config:
                        print(f"\n成功从 {DEFAULT_BUILTIN_CONFIG_PATH} 加载默认配置")
                        return default_config
            except Exception as e:
                print(f"Error loading default config file {DEFAULT_BUILTIN_CONFIG_PATH}: {e}")
                raise RuntimeError(f"加载默认配置文件失败: {e}")

        # 没有任何可用配置，直接报错
        raise FileNotFoundError(
            f"未找到配置文件: {self.config_file or DEFAULT_CONFIG_PATH}，也未找到默认内置配置: {DEFAULT_BUILTIN_CONFIG_PATH}\n"
            f"请先运行 'nezha init' 进行初始化！"
        )

    def plan_chat(self, history: list, verbose: bool = False, stream: bool = False):
        """Handles the interactive planning chat loop.
        
        Args:
            history: 对话历史记录
            verbose: 是否显示详细日志
            stream: 是否使用流式输出
            
        Returns:
            如果stream=True，返回生成器；否则返回字符串
        """
        if verbose and not stream:
            print(f"\n--- Sending History to LLM ({self.config.get('llm', {}).get('provider')}) ---")
            for msg in history:
                print(f"[{msg['role']}]: {msg['content']}")
            print("---------------------------------------")

        try:
            # Directly use the history provided by PlanCommand
            response = self.llm_interface.chat(history, stream=stream)
            # if verbose:
            #     print("\n--- LLM Response ---")
            #     print(response)
            #     print("--------------------")
            return response
        except Exception as e:
            error_msg = f"Error during planning chat: {e}"
            print(f"Error during LLM call in plan_chat: {e}")
            if stream:
                # 流式输出模式下，将错误消息包装为生成器返回
                def error_generator():
                    yield error_msg
                return error_generator()
            return error_msg

    def run(self, prompt: str, context: Optional[Dict[str, Any]] = None, verbose: bool = False, stream: bool = False):
        """执行用户指令
        
        Args:
            prompt: 用户输入的指令
            context: 上下文信息
            verbose: 是否显示详细日志
            stream: 是否使用流式输出
            
        Returns:
            如果stream=True，返回生成器；否则返回字符串
        """
        # TODO: Implement the core agent loop:
        # 1. Construct the full prompt including context, system prompt, etc.
        # 2. Call the LLM using self.llm_interface.chat() or generate()
        # 3. Parse the LLM response (e.g., identify tool calls)
        # 4. Execute tools securely using self.security_manager
        # 5. Format and return the final result

        # Placeholder implementation:
        full_prompt = f"Context:\n{context}\n\nUser Prompt: {prompt}"
        if verbose and not stream:
            print(f"\n--- Sending to LLM ({self.config.get('llm', {}).get('provider')}) ---")
            print(full_prompt)
            print("---------------------------------------")

        try:
            # Assuming chat interface is preferred
            # Construct messages list based on LLM provider requirements
            messages = [
                {"role": "system", "content": "You are a helpful AI assistant." }, # TODO: Load system prompt from config/prompts
                {"role": "user", "content": full_prompt}
            ]
            response = self.llm_interface.chat(messages, stream=stream)
            # if verbose:
            #     print("\n--- LLM Response ---")
            #     print(response)
            #     print("--------------------")
            return response
        except Exception as e:
            # TODO: More specific error handling
            # print(f"Error during LLM call: {e}")
            error_msg = f"Error executing prompt: {e}"
            if stream:
                # 流式输出模式下，将错误消息包装为生成器返回
                def error_generator():
                    yield error_msg
                return error_generator()
            return error_msg


"""
Nezha Agent 核心包

提供智能代理、上下文引擎、LLM接口、安全管理等核心组件
"""

from .features.agent.agent import NezhaAgent
from .core.security.security import SecurityManager, SecurityLevel
from .core.context.context_engine import ContextEngine
from .core.models.llm_interface import get_llm_interface, LLMInterfaceBase
from .features.commands.chat_command import ChatCommand
from .features.commands.plan_command import PlanCommand

__all__ = [
    'NezhaAgent',
    'SecurityManager',
    'SecurityLevel',
    'ContextEngine',
    'get_llm_interface',
    'LLMInterfaceBase',
    'ChatCommand',
    'PlanCommand',
]



"""
文件搜索与内容查找工具
"""
import glob
import re

from .base import BaseTool


class Glob(BaseTool):
    name = "Glob"
    description = "在指定目录查找匹配模式的文件/目录"
    arguments = {"pattern": "glob 匹配模式", "cwd": "搜索目录"}
    def execute(self, pattern, cwd="."):
        return glob.glob(pattern, root_dir=cwd)

class Grep(BaseTool):
    name = "Grep"
    description = "在文件中搜索内容"
    arguments = {"pattern": "正则表达式", "path": "文件路径"}
    def execute(self, pattern, path):
        try:
            with open(path, "r", encoding="utf-8") as f:
                lines = f.readlines()
            matches = [line for line in lines if re.search(pattern, line)]
            return matches
        except Exception as e:
            return f"搜索出错: {e}"



"""
Agent工具调用与对话流闭环实现
"""
from ...core.tools.tool_registry import run_tool
from typing import Any, Dict, Optional


def parse_tool_call(llm_response: str) -> Optional[Dict[str, Any]]:
    """
    简单解析LLM输出中的工具调用意图。
    约定格式: 
    工具调用: <tool=ToolName> <参数1=值1> <参数2=值2>
    例如: 工具调用: <tool=FileRead> <path=README.md>
    """
    import re
    pattern = r"工具调用:\s*<tool=(\w+)>((?:\s*<\w+=.*?>)*)"
    match = re.search(pattern, llm_response)
    if not match:
        return None
    tool_name = match.group(1)
    args_str = match.group(2)
    arg_pattern = r"<(\w+)=(.*?)>"
    args = dict(re.findall(arg_pattern, args_str))
    return {"tool_name": tool_name, "args": args}


def agent_with_tool_loop(agent, messages, verbose=False):
    """
    LLM回复如包含工具调用指令，则自动调用工具并追加结果到对话流。
    """
    history = messages[:]
    while True:
        response = agent.llm_interface.chat(history)
        if verbose:
            print("\n[LLM回复]:", response)
        tool_call = parse_tool_call(response)
        if tool_call:
            tool_result = run_tool(tool_call["tool_name"], tool_call["args"])
            tool_msg = f"[工具 {tool_call['tool_name']} 结果]:\n{tool_result}"
            history.append({"role": "function", "content": tool_msg})
            if verbose:
                print(f"\n[自动调用工具: {tool_call['tool_name']}]\n{tool_result}")
            # 继续循环，将工具结果反馈给LLM
            continue
        else:
            return response

# 用法示例：
# result = agent_with_tool_loop(agent, messages, verbose=True)



"""
Typer CLI 定义
"""
import os
import yaml
from pathlib import Path
from typing import Dict, List, Optional             

# 工具函数：自动补全 LLM 配置，确保 provider/api_key/endpoint 存在
def merge_llm_config(config: dict) -> dict:
    """
    根据 config['llm']['model']，自动补全 provider/api_key/endpoint 等字段。
    """
    llm_config = dict(config.get("llm", {}))
    models = config.get("models", [])
    model_id = llm_config.get("model")
    if model_id and isinstance(models, list):
        for m in models:
            if m.get("id") == model_id:
                for k in ["provider", "api_key", "endpoint"]:
                    if k in m and not llm_config.get(k):
                        llm_config[k] = m[k]
                break
    return llm_config

try:
    from platformdirs import user_config_dir
except ImportError:
    user_config_dir = None  # 兼容未安装 platformdirs 的情况

import typer
from rich.console import Console
from rich.markdown import Markdown
from rich.panel import Panel
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.prompt import Prompt
from rich.syntax import Syntax
from rich.table import Table

from ...features.agent.agent import NezhaAgent
from ...core.context.context_engine import ContextEngine
from ...features.commands.plan_command import PlanCommand
from ...core.security.security import SecurityLevel, SecurityManager

app = typer.Typer(
    help="nezha - AI 命令行代码助手\n\n模型管理相关命令：\n  nezha models              查看所有模型并切换当前模型\n  nezha models add          添加新模型到配置文件\n  nezha models list         仅列出所有模型（只读）\n\n其他命令请用 nezha --help 查看。",
    no_args_is_help=True,
    add_completion=True,
)

console = Console()

models_app = typer.Typer(help="模型管理相关命令")
app.add_typer(models_app, name="models", help="模型管理相关命令")

# 全局变量用于存储当前选择的模型
CURRENT_MODEL = None

# 预定义的模型列表
PREDEFINED_MODELS = [
    {
        "id": "ep-20250417174840-6c94l",
        "name": "火山引擎 - Doubao-1.5-pro-32k",
        "provider": "volcengine",
        "api_key": "****",
        "endpoint": "https://ark.cn-beijing.volces.com/api/v3",
    },
    {
        "id": "qwen3-235b-a22b",
        "model": "qwen3-235b-a22b",
        "name": "阿里千问 - Qwen3-235B",
        "provider": "qwen",
        "api_key": "****",
        "endpoint": "https://dashscope.aliyuncs.com/compatible-mode/v1",
    },
    {
        "id": "qwen-plus",
        "model": "qwen-plus-2025-04-28",
        "name": "阿里千问 - Qwen-Plus",
        "provider": "qwen",
        "api_key": "****",
        "endpoint": "https://dashscope.aliyuncs.com/compatible-mode/v1",
    }
]

# 获取当前选择的模型
def get_current_model():
    """获取当前选择的模型配置。如果没有设置，返回 None。"""
    global CURRENT_MODEL
    return CURRENT_MODEL

# 设置当前选择的模型
def set_current_model(model):
    """设置当前选择的模型配置。"""
    global CURRENT_MODEL
    CURRENT_MODEL = model

@models_app.command("list")
def list_models():
    """仅列出所有模型（只读）"""
    models(
        set_model=False
    )

@models_app.command("select")
def select_model():
    """交互式选择并设置默认模型（推荐使用）"""
    from rich.console import Console
    console = Console()
    # 复用 models 逻辑，进入交互选择模式
    models(set_model=True)


@models_app.command("add")
def add_model():
    """添加新模型到 config.yaml 的 models 列表"""
    models_add()

@models_app.command("__default__")
def models(
    set_model: bool = typer.Option(False, "--set", "-s", help="设置默认模型"),
):
    """列出可用的模型并允许用户选择"""
    config_path = get_user_config_path()
    config = {}
    if config_path.exists():
        with open(config_path, 'r', encoding='utf-8') as f:
            config = yaml.safe_load(f) or {}
    user_models = config.get("models", [])
    current_model_id = config.get("llm", {}).get("model")

    # 合并展示：预置模型 + 用户模型
    all_models = PREDEFINED_MODELS + user_models

    table = Table(title="可用的模型")
    table.add_column("序号", style="cyan")
    table.add_column("模型名称", style="green")
    table.add_column("提供商", style="yellow")
    table.add_column("模型 ID", style="blue")
    table.add_column("状态", style="magenta")

    for i, model in enumerate(all_models):
        model_id = model.get("id")
        status = "✓ 当前" if model_id == current_model_id else ""
        table.add_row(
            str(i+1),
            model.get("name", "未命名"),
            model.get("provider", "未知"),
            model_id or "未知",
            status
        )
    
    console.print(table)
    
    if set_model:
        # 交互式选择模型
        choice = Prompt.ask(
            "请选择要使用的模型 [序号]",
            choices=[str(i+1) for i in range(len(all_models))],
            default="1"
        )
        
        try:
            selected_index = int(choice) - 1
            selected_model = all_models[selected_index]
            
            # 更新配置文件
            if not config.get("llm"):
                config["llm"] = {}
            
            # 完全更新llm部分的配置
            config["llm"] = {
                "model": selected_model["id"],
                "provider": selected_model.get("provider"),
                "api_key": selected_model.get("api_key"),
            }
            
            # 如果有endpoint，也添加到配置中
            if "endpoint" in selected_model:
                config["llm"]["endpoint"] = selected_model["endpoint"]
            
            # 确保 models 列表中包含选中的模型
            if selected_index < len(PREDEFINED_MODELS):
                # 选择的是预定义模型，确保添加到用户配置
                model_exists = False
                for m in user_models:
                    if m.get("id") == selected_model["id"]:
                        model_exists = True
                        break
                
                if not model_exists:
                    if "models" not in config:
                        config["models"] = []
                    config["models"].append(selected_model)
            
            # 保存配置
            with open(config_path, 'w', encoding='utf-8') as f:
                yaml.dump(config, f, default_flow_style=False, sort_keys=False, allow_unicode=True)
            
            console.print(f"[bold green]✓[/bold green] 已将默认模型设置为: [bold]{selected_model.get('name')}[/bold]")
            
            # 设置当前模型（全局变量）
            set_current_model(selected_model)
            
        except (ValueError, IndexError):
            console.print("[bold red]✗[/bold red] 无效的选择")
            return

@models_app.command("add")
def models_add():
    """添加新模型到 config.yaml 的 models 列表"""
    config_path = get_user_config_path()
    config = {}
    if config_path.exists():
        with open(config_path, 'r', encoding='utf-8') as f:
            config = yaml.safe_load(f) or {}
    
    # 获取用户输入
    console.print("[bold]添加新模型[/bold]")
    name = Prompt.ask("模型名称")
    provider = Prompt.ask("提供商", choices=["openai", "volcengine", "other"], default="openai")
    if provider == "other":
        provider = Prompt.ask("请输入提供商名称")
    
    model = Prompt.ask("模型名称/ID")
    api_key = Prompt.ask("API Key")
    endpoint = Prompt.ask("API 端点", default="")
    
    # 创建模型配置
    new_model = {
        "id": model,  # 使用模型名称作为内部ID
        "model": model,  # 同时设置 model 字段
        "name": name,
        "provider": provider,
        "api_key": api_key
    }
    
    if endpoint:
        new_model["endpoint"] = endpoint
    
    # 更新配置
    if "models" not in config:
        config["models"] = []
    
    config["models"].append(new_model)
    
    # 保存配置
    with open(config_path, 'w', encoding='utf-8') as f:
        yaml.dump(config, f, default_flow_style=False, sort_keys=False, allow_unicode=True)
    
    console.print(f"[bold green]✓[/bold green] 已添加新模型: [bold]{name}[/bold]")

@app.command()
def main(
    prompt: str = typer.Argument(..., help="输入你的自然语言指令"),
    verbose: bool = typer.Option(False, "--verbose", "-v", help="显示详细执行信息"),
    stream: bool = typer.Option(True, "--stream", help="启用流式输出（默认开启）"),
    security_level: str = typer.Option(
        "normal", "--security", "-s",
        help="安全级别: strict(严格), normal(普通), relaxed(宽松), bypass(绕过)"
    ),
    yes_to_all: bool = typer.Option(False, "--yes", "-y", help="自动确认所有操作"),
    config_file: Optional[Path] = typer.Option(None, "--config", help="配置文件路径"),
    api_key: Optional[str] = typer.Option(None, "--api-key", help="通过命令行注入大模型 API Key，优先级高于配置文件和环境变量")
):
    """nezha 主命令入口 - 执行用户给出的任务指令"""
    # 默认读取用户级别配置
    if config_file is None:
        config_file = get_user_config_path()
    
    # 显示任务信息
    console.print(Panel(f"[bold]执行任务:[/bold] {prompt}", title="nezha", border_style="blue"))
    
    # 将字符串安全级别转换为枚举，防止类型混用
    security_level_map = {
        "strict": SecurityLevel.STRICT,
        "normal": SecurityLevel.NORMAL,
        "relaxed": SecurityLevel.RELAXED,
        "bypass": SecurityLevel.BYPASS
    }
    if security_level.lower() not in security_level_map:
        raise typer.BadParameter(f"不支持的安全级别: {security_level}，可选值: strict, normal, relaxed, bypass")
    security_enum = security_level_map[security_level.lower()]
    # 初始化安全管理器，后续所有用到安全等级的地方都只能用 security_enum
    security_manager = SecurityManager(security_enum, yes_to_all=yes_to_all)
    
    try:
        # 初始化上下文引擎
        context_engine = ContextEngine(working_dir=os.getcwd())
        
        # 收集上下文信息
        context_engine.collect()
        
        # 初始化 Agent
        agent = NezhaAgent(
            security_manager=security_manager,
            config_file=config_file,
            api_key=api_key
        )
        
        # 设置上下文引擎
        agent.context_engine = context_engine
        
        if not stream:
            with Progress(
                SpinnerColumn(),
                TextColumn("[bold blue]{task.description}"),
                console=console,
                transient=True,
            ) as progress:
                progress.add_task("正在加载模型...", total=None)
                # 执行任务
                result = agent.run(prompt)
                # 兼容 agent.run 返回字符串，包装成 dict
                if not isinstance(result, dict):
                    result = {"response": result, "error": None, "tool_calls": []}
        else:
            # 流式输出模式
            console.print("\n[bold cyan]nezha:[/bold cyan]")
            console.print("[dim](按Ctrl+C可中断生成)[/dim]")
            # 执行任务并获取生成器
            response_generator = agent.run(prompt, verbose=verbose, stream=True)
            # 逐个输出响应片段
            response_text = ""
            try:
                for chunk in response_generator:
                    console.print(chunk, end="")
                    response_text += chunk
            except KeyboardInterrupt:
                console.print("\n\n[bold red][生成已被用户中断][/bold red]")
            finally:
                console.print()  # 换行
                # 包装结果
                result = {"response": response_text, "error": None, "tool_calls": []}
        
        # 显示结果
        if result.get("error"):
            console.print(Panel(f"[bold]执行出错:[/bold] {result['error']}", title="错误", border_style="red"))
        elif not stream:  # 只在非流式输出模式下显示完整响应
            # 显示 AI 回复
            console.print("\n[bold cyan]nezha:[/bold cyan]")
            
            # 尝试解析 Markdown
            try:
                markdown_content = result.get("response", "")
                console.print(Markdown(markdown_content))
            except (ValueError, TypeError) as error:
                # 如果解析失败，直接显示原始文本
                console.print(result.get("response", ""))
            
            # 显示执行的工具调用
            if verbose and result.get("tool_calls"):
                console.print("\n[bold yellow]执行的工具调用:[/bold yellow]")
                for i, call in enumerate(result.get("tool_calls", [])):
                    tool_name = call.get("name", "未知工具")
                    tool_args = call.get("arguments", {})
                    console.print(f"[bold]{i+1}.[/bold] [cyan]{tool_name}[/cyan]")
                    
                    # 格式化显示参数
                    args_syntax = Syntax(
                        yaml.dump(tool_args, default_flow_style=False, sort_keys=False, allow_unicode=True),
                        "yaml",
                        theme="monokai",
                        line_numbers=False,
                    )
                    console.print(args_syntax)
                    
                    # 显示结果
                    if "result" in call:
                        result_syntax = Syntax(
                            str(call["result"]),
                            "text",
                            theme="monokai",
                            line_numbers=False,
                        )
                        console.print(Panel(result_syntax, title="结果", border_style="green"))
    
    except (ValueError, TypeError) as error:
        # 处理可预期的错误类型
        console.print(Panel(f"[bold]执行任务时出错:[/bold] {error}", title="错误", border_style="red"))
        if verbose:
            import traceback
            console.print(traceback.format_exc())
        raise typer.Exit(code=1)
    except Exception as error:
        # 处理未预期的错误
        console.print(Panel(f"[bold]执行任务时出错:[/bold] {error}", title="错误", border_style="red"))
        if verbose:
            import traceback
            console.print(traceback.format_exc())
        raise typer.Exit(code=1)

@app.command()
def plan(
    initial_requirement: str = typer.Argument(..., help="初始需求描述"),
    verbose: bool = typer.Option(False, "--verbose", "-v", help="显示详细执行信息"),
    stream: bool = typer.Option(True, "--stream", help="启用流式输出（默认开启）"),
    security_level: str = typer.Option(
        "normal", "--security", "-s",
        help="安全级别: strict(严格), normal(标准), relaxed(宽松), bypass(绕过)"
    ),
    config_file: Optional[Path] = typer.Option(None, "--config", help="配置文件路径"),
    api_key: Optional[str] = typer.Option(None, "--api-key", help="通过命令行注入大模型 API Key，优先级高于配置文件和环境变量")
):
    """nezha 规划命令入口 - 通过交互式对话生成任务计划"""
    # 默认读取用户级别配置
    if config_file is None:
        config_file = get_user_config_path()
    
    # 显示任务信息
    console.print(Panel(f"[bold]需求描述:[/bold] {initial_requirement}", title="nezha plan", border_style="blue"))
    
    try:
        # 将字符串安全级别转换为枚举类型
        security_level_map = {
            "strict": SecurityLevel.STRICT,
            "normal": SecurityLevel.NORMAL,
            "relaxed": SecurityLevel.RELAXED,
            "bypass": SecurityLevel.BYPASS
        }
        if security_level.lower() not in security_level_map:
            raise typer.BadParameter(f"不支持的安全级别: {security_level}，可选值: strict, normal, relaxed, bypass")
        security_enum = security_level_map[security_level.lower()]
        
        # 初始化安全管理器
        security_manager = SecurityManager(security_enum)
        
        # 初始化上下文引擎
        context_engine = ContextEngine(working_dir=os.getcwd())
        
        # 收集上下文信息
        context_engine.collect()
        
        # 初始化 Agent
        agent = NezhaAgent(
            security_manager=security_manager,
            config_file=config_file,
            api_key=api_key
        )
        
        # 设置上下文引擎
        agent.context_engine = context_engine
        
        # 初始化 PlanCommand
        plan_command = PlanCommand(
            agent=agent,
            context_engine=context_engine,
            verbose=verbose,
            stream=stream
        )
        
        # 在非流式输出模式下显示加载进度
        if not stream:
            with Progress(
                SpinnerColumn(),
                TextColumn("[bold blue]{task.description}"),
                console=console,
                transient=True,
            ) as progress:
                progress.add_task("正在加载模型...", total=None)
                # 执行规划
                plan_command.run(initial_requirement)
        else:
            # 流式输出模式下直接执行规划
            plan_command.run(initial_requirement)
    
    except (ValueError, TypeError) as error:
        # 处理可预期的错误类型
        console.print(Panel(f"[bold]执行规划时出错:[/bold] {error}", title="错误", border_style="red"))
        if verbose:
            import traceback
            console.print(traceback.format_exc())
        raise typer.Exit(code=1)
    except Exception as error:
        # 处理未预期的错误
        console.print(Panel(f"[bold]执行规划时出错:[/bold] {error}", title="错误", border_style="red"))
        if verbose:
            import traceback
            console.print(traceback.format_exc())
        raise typer.Exit(code=1)


def version_callback(value: bool):
    if value:
        console.print("[bold cyan]nezha[/bold cyan] v0.1.0")
        raise typer.Exit()

# 获取用户级别的 nezha 配置文件路径 (~/.config/nezha/config.yaml)
def get_user_config_path():
    """获取用户级别的 nezha 配置文件路径 (~/.config/nezha/config.yaml)"""
    if user_config_dir:
        config_dir = Path(user_config_dir("nezha", "nezha"))
    else:
        # 兼容未安装 platformdirs 的情况
        config_dir = Path.home() / ".config" / "nezha"
    config_dir.mkdir(parents=True, exist_ok=True)
    return config_dir / "config.yaml"

# 获取用户级别的 nezha 安全配置文件路径 (~/.config/nezha/security_config.yaml)
def get_user_security_config_path():
    """获取用户级别的 nezha 安全配置文件路径 (~/.config/nezha/security_config.yaml)"""
    if user_config_dir:
        config_dir = Path(user_config_dir("nezha", "nezha"))
    else:
        # 兼容未安装 platformdirs 的情况
        config_dir = Path.home() / ".config" / "nezha"
    config_dir.mkdir(parents=True, exist_ok=True)
    return config_dir / "security_config.yaml"

@app.command()
def init(
    config_file: Optional[Path] = typer.Option(
        None, "--config", "-c", help="配置文件路径，默认写入用户目录 ~/.config/nezha/config.yaml"
    ),
    security_config: Optional[Path] = typer.Option(
        None, "--security-config", "-s", help="安全配置文件路径，默认写入用户目录 ~/.config/nezha/security_config.yaml"
    ),
    api_key: Optional[str] = typer.Option(
        None, "--api-key", help="通过命令行注入大模型 API Key，优先级高于配置文件和环境变量"
    ),
):
    """nezha 初始化命令 - 配置大模型接口、token和规则集"""
    # 默认读取用户级别配置
    if config_file is None:
        config_file = get_user_config_path()
    
    if security_config is None:
        security_config = get_user_security_config_path()
    
    # 确保目录存在
    config_file.parent.mkdir(parents=True, exist_ok=True)
    security_config.parent.mkdir(parents=True, exist_ok=True)
    
    console.print(Panel("[bold]欢迎使用 nezha 初始化向导[/bold]", title="nezha init", border_style="blue"))
    console.print("这将帮助你配置 nezha 的基本设置，包括大模型接口和安全级别。")
    
    # 读取已有配置（如果存在）
    existing_config = {}
    if config_file.exists():
        try:
            with open(config_file, "r", encoding="utf-8") as f:
                existing_config = yaml.safe_load(f) or {}
            console.print(f"[bold yellow]注意:[/bold yellow] 检测到已有配置文件 {config_file}，将在其基础上更新。")
        except (IOError, yaml.YAMLError) as error:
            console.print(f"[bold red]✗[/bold red] 读取现有配置文件失败: {error}")
    
    # 选择模型提供商
    providers = ["openai", "volcengine", "other"]
    provider_names = {
        "openai": "OpenAI API",
        "volcengine": "火山引擎",
        "other": "其他提供商"
    }
    
    console.print("\n[bold]第一步: 选择大模型提供商[/bold]")
    for i, p in enumerate(providers):
        console.print(f"  {i+1}. {provider_names[p]}")
    
    provider_idx = Prompt.ask(
        "请选择提供商 [序号]",
        choices=[str(i+1) for i in range(len(providers))],
        default="1"
    )
    provider = providers[int(provider_idx) - 1]
    
    if provider == "other":
        provider = Prompt.ask("请输入提供商名称")
    
    # 配置 API Key
    console.print(f"\n[bold]第二步: 配置 {provider_names.get(provider, provider)} API Key[/bold]")
    
    # 如果命令行提供了 API Key，优先使用
    if api_key:
        console.print(f"[bold green]✓[/bold green] 已通过命令行参数提供 API Key")
    else:
        # 尝试从环境变量获取
        env_var_name = f"{provider.upper()}_API_KEY"
        env_api_key = os.environ.get(env_var_name)
        
        if env_api_key:
            use_env = Prompt.ask(
                f"检测到环境变量 {env_var_name}，是否使用该值?",
                choices=["y", "n"],
                default="y"
            )
            
            if use_env.lower() == "y":
                api_key = env_api_key
                console.print(f"[bold green]✓[/bold green] 已使用环境变量中的 API Key")
            else:
                api_key = Prompt.ask("请输入 API Key", password=True)
        else:
            api_key = Prompt.ask("请输入 API Key", password=True)
    
    # 配置 API 端点
    endpoint = None
    if provider != "openai":
        console.print(f"\n[bold]第三步: 配置 {provider_names.get(provider, provider)} API 端点[/bold]")
        default_endpoints = {
            "volcengine": "https://ark.cn-beijing.volces.com/api/v3",
        }
        default_endpoint = default_endpoints.get(provider, "")
        endpoint = Prompt.ask("请输入 API 端点", default=default_endpoint)
    
    # 选择默认模型
    console.print("\n[bold]第四步: 选择默认模型[/bold]")
    default_models = {
        "openai": [
            {"id": "gpt-4", "name": "GPT-4"},
            {"id": "gpt-4-turbo", "name": "GPT-4 Turbo"},
            {"id": "gpt-3.5-turbo", "name": "GPT-3.5 Turbo"},
        ],
        "volcengine": [
            {"id": "ep-20250417174840-6c94l", "name": "Doubao-1.5-pro-32k"},
        ]
    }
    
    models = default_models.get(provider, [])
    if models:
        for i, m in enumerate(models):
            console.print(f"  {i+1}. {m['name']} ({m['id']})")
        
        model_idx = Prompt.ask(
            "请选择默认模型 [序号]",
            choices=[str(i+1) for i in range(len(models))],
            default="1"
        )
        model = models[int(model_idx) - 1]["id"]
    else:
        model = Prompt.ask("请输入模型 ID")
    
    # 选择安全级别
    console.print("\n[bold]第五步: 选择安全级别[/bold]")
    security_levels = [
        {"id": "strict", "name": "严格 (只允许读取文件)"},
        {"id": "normal", "name": "普通 (允许读写文件，需确认高风险操作)"},
        {"id": "relaxed", "name": "宽松 (允许执行 shell 命令，需确认高风险操作)"},
        {"id": "bypass", "name": "绕过 (允许所有操作，不需确认)"}
    ]
    
    for i, level in enumerate(security_levels):
        console.print(f"  {i+1}. {level['name']}")
    
    level_idx = Prompt.ask(
        "请选择安全级别 [序号]",
        choices=[str(i+1) for i in range(len(security_levels))],
        default="2"
    )
    security_level = security_levels[int(level_idx) - 1]["id"]
    
    # 构建 LLM 配置
    llm_config = {
        "provider": provider,
        "api_key": api_key,
        "model": model
    }
    
    if endpoint:
        llm_config["endpoint"] = endpoint
    
    # 生成主配置文件
    full_config = {
        "llm": llm_config,
        "security": {
            "allow_bash": security_level in ["relaxed", "bypass"],
            "allow_file_write": security_level != "strict",
            "allow_file_edit": security_level != "strict",
            "confirm_high_risk": security_level != "bypass"
        },
        "tools": {
            "enabled": [
                "FileRead", 
                "FileWrite", 
                "FileEdit", 
                "Glob", 
                "Grep", 
                "Ls"
            ]
        }
    }
    # 暂时注释掉 rules 配置相关代码，避免 use_rules 未定义错误
    # if use_rules:
    #     full_config["rules"] = rules_config

    # 生成安全配置文件
    security_config_data = {
        "security_level": security_level,
        "yes_to_all": False,
        "allowed_paths": [],
        "disabled_tools": []
    }
    # 写入配置文件
    try:
        with open(config_file, "w", encoding="utf-8") as f:
            yaml.dump(full_config, f, default_flow_style=False, sort_keys=False, allow_unicode=True)
        with open(security_config, "w", encoding="utf-8") as f:
            yaml.dump(security_config_data, f, default_flow_style=False, sort_keys=False, allow_unicode=True)
        console.print(f"\n[bold green]✓[/bold green] 配置已保存至: [bold]{config_file}[/bold] 和 [bold]{security_config}[/bold]")
        console.print("\n现在你可以使用 [bold]nezha <指令>[/bold] 来执行任务了!")
    except (IOError, yaml.YAMLError) as error:
        console.print(Panel(f"[bold]保存配置时出错:[/bold] {error}", title="错误", border_style="red"))
        raise typer.Exit(code=1)
    except Exception as error:
        console.print(Panel(f"[bold]未预期的错误:[/bold] {error}", title="错误", border_style="red"))
        raise typer.Exit(code=1)


@app.command()
def chat(
    initial_message: Optional[str] = typer.Argument(None, help="初始对话消息"),
    verbose: bool = typer.Option(False, "--verbose", "-v", help="显示详细执行信息"),
    stream: bool = typer.Option(True, "--stream", help="启用流式输出（默认开启）"),
    config_file: Optional[Path] = typer.Option(None, "--config", help="配置文件路径"),
    api_key: Optional[str] = typer.Option(None, "--api-key", help="通过命令行注入大模型 API Key，优先级高于配置文件和环境变量")
):
    """nezha 对话命令 - 与AI助手进行交互式对话"""
    # 默认读取用户级别配置
    if config_file is None:
        config_file = get_user_config_path()
    # 显示开始对话的信息
    console.print(Panel("[bold]开始与AI助手对话[/bold]", title="nezha chat", border_style="blue"))
    
    try:
        # ========== 启用多轮对话流程 ===========
        # 初始化安全管理器和 Agent
        # 注意：chat 命令暂时默认 normal，如需支持自定义传参可扩展参数
        security_level_map = {
            "strict": SecurityLevel.STRICT,
            "normal": SecurityLevel.NORMAL,
            "relaxed": SecurityLevel.RELAXED,
            "bypass": SecurityLevel.BYPASS
        }
        security_enum = security_level_map["normal"]  # 默认 normal
        security_manager = SecurityManager(security_enum)
        agent = NezhaAgent(security_manager=security_manager, config_file=config_file, api_key=api_key)
        
        # 导入并运行 ChatCommand
        from ...features.commands.chat_command import ChatCommand
        chat_cmd = ChatCommand(
            agent=agent,
            verbose=verbose,
            stream=stream
        )
        chat_cmd.run(initial_message)
        
    except (ValueError, TypeError) as error:
        console.print(Panel(f"[bold]执行对话时出错:[/bold] {error}", title="错误", border_style="red"))
        raise typer.Exit(code=1)
    except Exception as error:
        console.print(Panel(f"[bold]未预期的错误:[/bold] {error}", title="错误", border_style="red"))
        raise typer.Exit(code=1)


@app.callback(invoke_without_command=True)
def callback(
    ctx: typer.Context,
    version: bool = typer.Option(False, "--version", "-V", help="显示版本信息", callback=version_callback)
):
    """nezha - 基于AI的命令行代码助手"""
    # 只在没有子命令时显示欢迎信息
    if ctx.invoked_subcommand is None and not version:
        console.print(
            "[bold cyan]nezha[/bold cyan] - [italic]AI命令行代码助手[/italic] 🚀\n",
            "使用 [bold]nezha <指令>[/bold] 执行任务，[bold]nezha plan <需求>[/bold] 进行交互式规划，[bold]nezha chat[/bold] 进行对话，或 [bold]nezha init[/bold] 初始化配置\n"
        )
        console.print("运行 [bold]nezha --help[/bold] 获取更多帮助信息")

if __name__ == "__main__":
    app()



#!/usr/bin/env python
"""
PlanCommand单元测试
"""
import unittest
from pathlib import Path
from unittest.mock import MagicMock, patch

from src.agent import NezhaAgent
from src.context_engine import ContextEngine
from src.plan_command import PlanCommand


class TestPlanCommand(unittest.TestCase):
    """测试交互式规划命令"""
    
    def setUp(self):
        """测试前准备"""
        self.mock_agent = MagicMock(spec=NezhaAgent)
        self.mock_context_engine = MagicMock(spec=ContextEngine)
        self.mock_context_engine.collect.return_value = "测试上下文"
        
        # 模拟agent.plan_chat方法返回响应
        self.mock_agent.plan_chat.side_effect = [
            "我需要了解更多关于你的项目。你使用什么编程语言？",  # 第一次回复
            "明白了。我建议以下步骤:\n1. 设置pytest框架\n2. 创建基本测试\n3. 配置GitHub Actions\n\n这个计划可行吗？",  # 第二次回复
            "# 项目测试与CI/CD集成计划\n\n## 目标\n为项目添加单元测试框架和CI/CD集成\n\n## 详细步骤\n1. 安装pytest\n2. 创建测试目录结构\n3. 编写基本测试用例\n4. 配置GitHub Actions\n\n[计划完成]"  # 最终计划
        ]
        
        # 创建PlanCommand实例
        self.planner = PlanCommand(
            agent=self.mock_agent,
            context_engine=self.mock_context_engine,
            verbose=False,
            output_file=None
        )
    
    @patch('src.plan_command.input')
    @patch('src.plan_command.print')
    def test_run_plan_command(self, mock_print, mock_input):
        """测试运行交互式规划流程"""
        # 模拟用户输入
        mock_input.side_effect = [
            "我的项目使用Python",  # 第一次用户回复
            "可行，请生成详细计划",  # 第二次用户回复
            ""  # 用户回车结束
        ]
        
        # 执行规划
        result = self.planner.run("为项目添加单元测试框架和CI/CD集成")
        
        # 验证结果
        self.assertIn("# 项目测试与CI/CD集成计划", result)
        self.assertIn("## 目标", result)
        self.assertIn("## 详细步骤", result)
        
        # 验证方法调用
        self.assertEqual(self.mock_agent.plan_chat.call_count, 3) # 修正: 检查 plan_chat 而不是 run
        # self.assertEqual(self.mock_context_engine.collect.call_count, 1) # 移除: collect 未在 PlanCommand 中调用
        self.assertEqual(mock_input.call_count, 3) # 修正: 预期调用 3 次 input
    
    @patch('src.plan_command.open', new_callable=unittest.mock.mock_open) # 修正: 模拟内置 open 函数
    @patch('src.plan_command.print') # 修正: 目标是 print
    @patch('src.plan_command.input') # 修正: 目标是 input
    def test_output_file_saving(self, mock_input, mock_print, mock_open): # 修正: 参数名 mock_open
        """测试计划保存到文件"""
        # 设置输出文件
        self.planner.output_file = Path("test_plan.md")
        
        # 模拟用户输入
        mock_input.side_effect = ["", ""] # 修正: 模拟用户直接回车结束，因为 test_output_file_saving 不涉及多轮交互
        
        # 执行规划
        self.planner.run("测试需求")
        
        # 验证文件写入
        mock_open.assert_called_once_with(self.planner.output_file, "w", encoding="utf-8")
        mock_open().write.assert_called_once()

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
测试security模块是否正常工作
"""

from security import SecurityLevel, ToolRiskLevel, security_manager

print('导入成功！安全模块正常工作')
print(f'当前安全级别: {security_manager.security_level}')

# 测试风险级别
print(f'低风险级别: {ToolRiskLevel.LOW}')
print(f'中风险级别: {ToolRiskLevel.MEDIUM}')
print(f'高风险级别: {ToolRiskLevel.HIGH}')
print(f'极高风险级别: {ToolRiskLevel.CRITICAL}')